#include <iostream>
#include <vector>
#include <map>
#include <algorithm>

using namespace std;

const int INF = 2000000001;

struct Event {
    int x;
    int y1, y2;
    int type;

    Event(int x, int y1, int y2, int type) : x(x), y1(y1), y2(y2), type(type) {}
};

struct SegTree1 {
    int sz;
    vector<int> it, mod;

    void push(int u) {
        it[2 * u + 1] += mod[u];
        mod[2 * u + 1] += mod[u];
        it[2 * u + 2] += mod[u];
        mod[2 * u + 2] += mod[u];
        mod[u] = 0;
    }

    SegTree1(int sz) : sz(sz) {
        it.resize(4 * sz);
        mod.resize(4 * sz);
    }

    void add_(int u, int L, int R, int l, int r, int v) {
        if (l >= R || L >= r) {
            return;
        }
        if (l <= L && R <= r) {
            it[u] += v;
            mod[u] += v;
            return;
        }
        push(u);
        int M = (L + R) / 2;
        add_(2 * u + 1, L, M, l, r, v);
        add_(2 * u + 2, M, R, l, r, v);
        it[u] = min(it[2 * u + 1], it[2 * u + 2]);
    }

    inline void add(int l, int r, int v) {
        add_(0, 0, sz, l, r, v);
    }

    int get_(int u, int L, int R, int l, int r) {
        if (l >= R || L >= r) {
            return INF;
        }
        if (l <= L & R <= r) {
            return it[u];
        }
        push(u);
        int M = (L + R) / 2;
        return min(
                get_(2 * u + 1, L, M, l, r),
                get_(2 * u + 2, M, R, l, r)
        );
    }

    inline int get(int l, int r) {
        return get_(0, 0, sz, l, r);
    }
};

inline bool cmp(Event &e1, Event &e2) {
    if (e1.x != e2.x) {
        return e1.x < e2.x;
    }
    return e1.type < e2.type;
}

int r, c;
int n;
vector<int> x, y;

inline int get_y1(int x0) {
    return x0 <= c - 1 ? -x0 : x0 - 2 * (c - 1);
}

inline int get_y2(int x0) {
    return x0 <= r - 1 ? x0 : 2 * (r - 1) - x0;
}

vector<Event> evs;

inline void add_ev(int x, int y, int yy, int tp) {
    if (y <= yy) {
        evs.push_back(Event(x, y, yy, tp));
    }
}

void checkit(int x) {
    add_ev(x, get_y1(x), get_y2(x), 0); // check
    add_ev(x + 1, get_y1(x + 1), get_y2(x + 1), 0); // check
    add_ev(x - 1, get_y1(x - 1), get_y2(x - 1), 0); // check
}

bool check(int d) {
    evs.clear();

    for (int i = 0; i < n; i++) {
        int x1 = x[i] + y[i] - d, x2 = x[i] + y[i] + d;
        int y1 = x[i] - y[i] - d, y2 = x[i] - y[i] + d;
        evs.push_back(Event(x1, y1, y2, -1));
        evs.push_back(Event(x2, y1, y2, 1));

        checkit(x1);
        checkit(x2);
    }

    checkit(0);
    checkit(c - 1);
    checkit(r - 1);
    checkit(r + c - 2);

    sort(evs.begin(), evs.end(), cmp);

    vector<int> ys_odd, ys_even;
    for (auto &e : evs) {
        if (e.y1 & 1) {
            ys_odd.push_back(e.y1);
        } else {
            ys_even.push_back(e.y1);
        }
        if (e.y2 & 1) {
            ys_odd.push_back(e.y2);
        } else {
            ys_even.push_back(e.y2);
        }
    }

    sort(ys_odd.begin(), ys_odd.end());
    ys_odd.resize(unique(ys_odd.begin(), ys_odd.end()) - ys_odd.begin());

    sort(ys_even.begin(), ys_even.end());
    ys_even.resize(unique(ys_even.begin(), ys_even.end()) - ys_even.begin());

    SegTree1 st_odd(ys_odd.size()), st_even(ys_even.size());
    bool ok = true;
    for (auto &e : evs) {
        if (e.type == -1 || e.type == 1) {
            int l_odd = lower_bound(ys_odd.begin(), ys_odd.end(), e.y1) - ys_odd.begin();
            int r_odd = lower_bound(ys_odd.begin(), ys_odd.end(), e.y2 + 1) - ys_odd.begin();
            int l_even = lower_bound(ys_even.begin(), ys_even.end(), e.y1) - ys_even.begin();
            int r_even = lower_bound(ys_even.begin(), ys_even.end(), e.y2 + 1) - ys_even.begin();
            st_odd.add(l_odd, r_odd, -e.type);
            st_even.add(l_even, r_even, -e.type);
        } else {
            if (e.x & 1) {
                int l_odd = lower_bound(ys_odd.begin(), ys_odd.end(), e.y1) - ys_odd.begin();
                int r_odd = lower_bound(ys_odd.begin(), ys_odd.end(), e.y2 + 1) - ys_odd.begin();
                if (st_odd.get(l_odd, r_odd) == 0) {
                    return false;
                }
            } else {
                int l_even = lower_bound(ys_even.begin(), ys_even.end(), e.y1) - ys_even.begin();
                int r_even = lower_bound(ys_even.begin(), ys_even.end(), e.y2 + 1) - ys_even.begin();
                if (st_even.get(l_even, r_even) == 0) {
                    return false;
                }
            }

        }
    }

    return ok;
}

int main() {
#ifdef DEBUG
    freopen("input", "r", stdin);
#endif
    ios::sync_with_stdio(0);
    cin.tie(0);

    scanf("%d%d", &r, &c);
    scanf("%d", &n);
    x.resize(n);
    y.resize(n);

    for (int i = 0; i < n; i++) {
        scanf("%d%d", &x[i], &y[i]);
        x[i]--;
        y[i]--;
    }

    int L = -1;
    int R = r + c;
    while (R - L > 1) {
        int M = (L + R) / 2;
#ifdef DEBUG
        cerr << M << endl;
#endif
        if (check(M)) {
            R = M;
        } else {
            L = M;
        }
    }

    cout << R << endl;

#ifdef DEBUG
    cerr << (1.0 * clock()) / CLOCKS_PER_SEC << endl;
#endif

    return 0;
}
