#include <iostream>
#include <vector>
#include <map>
#include <algorithm>

using namespace std;

const int INF = 2000000001;

struct Event {
    long long x;
    long long y1, y2;
    int type;

    Event(long long int x, long long int y1, long long int y2, int type) : x(x), y1(y1), y2(y2), type(type) {}
};

struct SegTree {
    int sz;
    vector<int> it_odd, it_even;
    vector<bool> has_odd, has_even;
    vector<int> mod;

    void push(int u) {
        if (has_odd[2 * u + 1]) {
            it_odd[2 * u + 1] += mod[u];
        }
        if (has_odd[2 * u + 2]) {
            it_odd[2 * u + 2] += mod[u];
        }
        if (has_even[2 * u + 1]) {
            it_even[2 * u + 1] += mod[u];
        }
        if (has_even[2 * u + 2]) {
            it_even[2 * u + 2] += mod[u];
        }
        mod[2 * u + 1] += mod[u];
        mod[2 * u + 2] += mod[u];
        mod[u] = 0;
    }

    void update(int u) {
        it_odd[u] = min(it_odd[2 * u + 1], it_odd[2 * u + 2]);
        it_even[u] = min(it_even[2 * u + 1], it_even[2 * u + 2]);
    }

    void init(int u, int L, int R, vector<long long> &ys) {
        if (R - L == 1) {
            if (ys[L] & 1) {
                has_odd[u] = true;
                it_odd[u] = 0;
                it_even[u] = INF;
            } else {
                has_even[u] = true;
                it_even[u] = 0;
                it_odd[u] = INF;
            }
            return;
        }
        int M = (L + R) / 2;
        init(2 * u + 1, L, M, ys);
        init(2 * u + 2, M, R, ys);
        has_odd[u] = has_odd[2 * u + 1] || has_odd[2 * u + 2];
        has_even[u] = has_even[2 * u + 1] || has_even[2 * u + 2];
        update(u);
    }

    SegTree(int sz, vector<long long> &ys) : sz(sz) {
        mod.resize(4 * sz);
        it_odd.resize(4 * sz);
        it_even.resize(4 * sz);
        has_odd.resize(4 * sz);
        has_even.resize(4 * sz);
        init(0, 0, sz, ys);
    }

    void add_(int u, int L, int R, int l, int r, int v) {
        if (L >= r || l >= R) {
            return;
        }
        if (l <= L && R <= r) {
            it_odd[u] += v;
            it_even[u] += v;
            mod[u] += v;
            return;
        }
        push(u);
        int M = (L + R) / 2;
        add_(2 * u + 1, L, M, l, r, v);
        add_(2 * u + 2, M, R, l, r, v);
        update(u);
    }

    void add(int l, int r, int v) {
        add_(0, 0, sz, l, r + 1, v);
    }

    int min_odd_(int u, int L, int R, int l, int r) {
        if (l >= R || L >= r) {
            return INF;
        }
        if (l <= L && R <= r) {
            return it_odd[u];
        }
        push(u);
        int M = (L + R) / 2;
        return min(
                min_odd_(2 * u + 1, L, M, l, r),
                min_odd_(2 * u + 2, M, R, l, r)
        );
    }

    int min_odd(int l, int r) {
        return min_odd_(0, 0, sz, l, r + 1);
    }

    int min_even_(int u, int L, int R, int l, int r) {
        if (l >= R || L >= r) {
            return INF;
        }
        if (l <= L && R <= r) {
            return it_even[u];
        }
        push(u);
        int M = (L + R) / 2;
        return min(
                min_even_(2 * u + 1, L, M, l, r),
                min_even_(2 * u + 2, M, R, l, r)
        );
    }

    int min_even(int l, int r) {
        return min_even_(0, 0, sz, l, r + 1);
    }
};

inline bool cmp(Event &e1, Event &e2) {
    if (e1.x != e2.x) {
        return e1.x < e2.x;
    }
    return e1.type < e2.type;
}

int r, c;
int n;
vector<int> x, y;

long long get_y1(long long x0) {
    return x0 <= c - 1 ? -x0 : x0 - 2 * (c - 1);
}

long long get_y2(long long x0) {
    return x0 <= r - 1 ? x0 : 2 * (r - 1) - x0;
}

vector<Event> evs;

inline void add_ev(long long x, long long y, long long yy, int tp) {
    if (y <= yy) {
        evs.push_back(Event(x, y, yy, tp));
    }
}

void checkit(long long x) {
    add_ev(x, get_y1(x), get_y2(x), 0); // check
    add_ev(x + 1, get_y1(x + 1), get_y2(x + 1), 0); // check
    add_ev(x - 1, get_y1(x - 1), get_y2(x - 1), 0); // check
}

bool check(long long d) {
    evs.clear();

    for (int i = 0; i < n; i++) {
        long long x1 = x[i] + y[i] - d, x2 = x[i] + y[i] + d;
        long long y1 = x[i] - y[i] - d, y2 = x[i] - y[i] + d;
        evs.push_back(Event(x1, y1, y2, -1)); // start
        evs.push_back(Event(x2, y1, y2, 1)); // finish

        checkit(x1);
        checkit(x2);
    }

    checkit(0);
    checkit(c - 1);
    checkit(r - 1);
    checkit(r + c - 2);

    sort(evs.begin(), evs.end(), cmp);

    vector<long long> ys;
    for (auto &e : evs) {
        ys.push_back(e.y1);
        ys.push_back(e.y2);
    }
    sort(ys.begin(), ys.end());
    ys.resize(unique(ys.begin(), ys.end()) - ys.begin());

    map<long long, int> yr;
    for (int i = 0; i < ys.size(); i++) {
        yr[ys[i]] = i;
    }

    for (auto &e : evs) {
        e.y1 = yr[e.y1];
        e.y2 = yr[e.y2];
    }

    SegTree st(ys.size(), ys);
    for (auto &e : evs) {
        if (e.type == -1) {
            st.add(e.y1, e.y2, 1);
        } else if (e.type == 1) {
            st.add(e.y1, e.y2, -1);
        } else {
            if (e.x & 1) {
                if (st.min_odd(e.y1, e.y2) == 0) {
                    return false;
                }
            } else {
                if (st.min_even(e.y1, e.y2) == 0) {
                    return false;
                }
            }

        }
    }

    return true;
}

int main() {
#ifdef DEBUG
    freopen("input", "r", stdin);
#endif
    ios::sync_with_stdio(0);
    cin.tie(0);

    cin >> r >> c;
    cin >> n;
    x.resize(n);
    y.resize(n);

    for (int i = 0; i < n; i++) {
        cin >> x[i] >> y[i];
        x[i]--;
        y[i]--;
    }

    long long L = -1;
    long long R = r + c;
    while (R - L > 1) {
        long long M = (L + R) / 2;
        if (check(M)) {
            R = M;
        } else {
            L = M;
        }
    }

    cout << R << endl;

    return 0;
}
