#include <iostream>
#include <vector>
#include <algorithm>

using namespace std;

const int INF = 2000000001;

struct Event {
    long long x;
    long long y1, y2;
    int type;

    Event(long long int x, long long int y1, long long int y2, int type) : x(x), y1(y1), y2(y2), type(type) {}
};

struct SegTree {
    int sz;
    vector<int> val;
    vector<long long> ys;

    SegTree(int sz, vector<long long> &ys_) : sz(sz) {
        val.resize(sz);
        ys = ys_;
    }

    void add(int l, int r, int v) {
        for (int i = l; i <= r; i++) {
            val[i] += v;
        }
    }

    int min_odd(int l, int r) {
        int mn = INF;
        for (int i = l; i <= r; i++) {
            if (ys[i] & 1) {
                mn = min(mn, val[i]);
            }
        }
        return mn;
    }

    int min_even(int l, int r) {
        int mn = INF;
        for (int i = l; i <= r; i++) {
            if (!(ys[i] & 1)) {
                mn = min(mn, val[i]);
            }
        }
        return mn;
    }
};

inline bool cmp(Event &e1, Event &e2) {
    if (e1.x != e2.x) {
        return e1.x < e2.x;
    }
    return e1.type < e2.type;
}

int r, c;
int n;
vector<int> x, y;

long long get_y1(long long x0) {
    return x0 <= c - 1 ? -x0 : x0 - 2 * (c - 1);
}

long long get_y2(long long x0) {
    return x0 <= r - 1 ? x0 : 2 * (r - 1) - x0;
}

vector<Event> evs;

inline void add_ev(long long x, long long y, long long yy, int tp) {
    if (y <= yy) {
        evs.push_back(Event(x, y, yy, tp));
    }
}

void checkit(long long x) {
    add_ev(x, get_y1(x), get_y2(x), 0); // check
    add_ev(x + 1, get_y1(x + 1), get_y2(x + 1), 0); // check
    add_ev(x - 1, get_y1(x - 1), get_y2(x - 1), 0); // check
}

bool check(long long d) {
    evs.clear();

    for (int i = 0; i < n; i++) {
        long long x1 = x[i] + y[i] - d, x2 = x[i] + y[i] + d;
        long long y1 = x[i] - y[i] - d, y2 = x[i] - y[i] + d;
        evs.push_back(Event(x1, y1, y2, -1)); // start
        evs.push_back(Event(x2, y1, y2, 1)); // finish

        checkit(x1);
        checkit(x2);
    }

    checkit(0);
    checkit(c - 1);
    checkit(r - 1);
    checkit(r + c - 2);

    sort(evs.begin(), evs.end(), cmp);

    long long mn = 0;
    long long mx = 0;
    for (auto &e : evs) {
        mn = min(mn, e.y1);
        mx = max(mx, e.y2);
    }

    for (auto &e : evs) {
        e.y1 -= mn;
        e.y2 -= mn;
    }
    mx -= mn;
    mx++;

    vector<long long> ys;
    for (int i = 0; i < mx; i++) {
        ys.push_back(i);
    }

    SegTree st(mx, ys);
    for (auto &e : evs) {
        if (e.type == -1) {
            st.add(e.y1, e.y2, 1);
        } else if (e.type == 1) {
            st.add(e.y1, e.y2, -1);
        } else {
            if ((e.x - mn) & 1) {
                if (st.min_odd(e.y1, e.y2) == 0) {
                    return false;
                }
            } else {
                if (st.min_even(e.y1, e.y2) == 0) {
                    return false;
                }
            }

        }
    }

    return true;
}

int main() {
#ifdef DEBUG
    freopen("input", "r", stdin);
#endif
    ios::sync_with_stdio(0);
    cin.tie(0);

    cin >> r >> c;
    cin >> n;
    x.resize(n);
    y.resize(n);

    for (int i = 0; i < n; i++) {
        cin >> x[i] >> y[i];
        x[i]--;
        y[i]--;
    }

    long long L = -1;
    long long R = r + c;
    while (R - L > 1) {
        long long M = (L + R) / 2;
        if (check(M)) {
            R = M;
        } else {
            L = M;
        }
    }

    cout << R << endl;

    return 0;
}
