#include <iostream>
#include <vector>
#include <algorithm>
#include <cstring>

using namespace std;

const int MAXN = 422000;
const int INF = 2000000001;

struct Event {
    int x;
    int y1, y2;
    int type;

    Event(int x, int y1, int y2, int type) : x(x), y1(y1), y2(y2), type(type) {}
};

int it[2][4 * (MAXN + 2)];
int mod[2][4 * (MAXN + 2)];
int sz[2];

inline void push(int i, int u) {
    it[i][2 * u + 1] += mod[i][u];
    mod[i][2 * u + 1] += mod[i][u];
    it[i][2 * u + 2] += mod[i][u];
    mod[i][2 * u + 2] += mod[i][u];
    mod[i][u] = 0;
}

inline void update(int i, int u) {
    it[i][u] = min(it[i][2 * u + 1], it[i][2 * u + 2]);
}

void add_(int i, int u, int L, int R, int l, int r, int v) {
    if (L >= r || l >= R) {
        return;
    }
    if (l <= L && R <= r) {
        it[i][u] += v;
        mod[i][u] += v;
        return;
    }
    int M = (L + R) / 2;
    push(i, u);
    add_(i, 2 * u + 1, L, M, l, r, v);
    add_(i, 2 * u + 2, M, R, l, r, v);
    update(i, u);
}

void add(int i, int l, int r, int v) {
    add_(i, 0, 0, sz[i], l, r, v);
}

int get_(int i, int u, int L, int R, int l, int r) {
    if (l >= R || L >= r) {
        return INF;
    }
    if (l <= L && R <= r) {
        return it[i][u];
    }
    int M = (L + R) / 2;
    push(i, u);
    return min(get_(i, 2 * u + 1, L, M, l, r), get_(i, 2 * u + 2, M, R, l, r));
}

int get(int i, int l, int r) {
    return get_(i, 0, 0, sz[i], l, r);
}

void init() {
    memset(it, 0, sizeof(it));
    memset(mod, 0, sizeof(mod));
}

inline bool cmp(Event &e1, Event &e2) {
    if (e1.x != e2.x) {
        return e1.x < e2.x;
    }
    return e1.type < e2.type;
}

int r, c;
int n;
vector<int> x, y;

inline int get_y1(int x0) {
    return x0 <= c - 1 ? -x0 : x0 - 2 * (c - 1);
}

inline int get_y2(int x0) {
    return x0 <= r - 1 ? x0 : 2 * (r - 1) - x0;
}

vector<Event> evs;

inline void add_ev(int x, int y, int yy, int tp) {
    if (y <= yy) {
        evs.emplace_back(x, y, yy, tp);
    }
}

inline void checkit(int x) {
    add_ev(x, get_y1(x), get_y2(x), 0);
    add_ev(x + 1, get_y1(x + 1), get_y2(x + 1), 0);
    add_ev(x - 1, get_y1(x - 1), get_y2(x - 1), 0);
}

vector<int> ys_odd, ys_even;

inline void add_all(int l, int r, int v) {
    int lo = (int) (lower_bound(ys_odd.begin(), ys_odd.end(), l) - ys_odd.begin());
    int le = (int) (lower_bound(ys_even.begin(), ys_even.end(), l) - ys_even.begin());
    int ro = (int) (lower_bound(ys_odd.begin(), ys_odd.end(), r + 1) - ys_odd.begin());
    int re = (int) (lower_bound(ys_even.begin(), ys_even.end(), r + 1) - ys_even.begin());

    add(0, le, re, v);
    add(1, lo, ro, v);
}

inline int get_odd(int l, int r) {
    int lo = (int) (lower_bound(ys_odd.begin(), ys_odd.end(), l) - ys_odd.begin());
    int ro = (int) (lower_bound(ys_odd.begin(), ys_odd.end(), r + 1) - ys_odd.begin());

    return get(1, lo, ro);
}

inline int get_even(int l, int r) {
    int le = (int) (lower_bound(ys_even.begin(), ys_even.end(), l) - ys_even.begin());
    int re = (int) (lower_bound(ys_even.begin(), ys_even.end(), r + 1) - ys_even.begin());

    return get(0, le, re);
}

bool check(int d) {
    evs.clear();

    for (int i = 0; i < n; i++) {
        int x1 = x[i] + y[i] - d, x2 = x[i] + y[i] + d;
        int y1 = x[i] - y[i] - d, y2 = x[i] - y[i] + d;
        evs.emplace_back(x1, y1, y2, -1);
        evs.emplace_back(x2, y1, y2, 1); // 2

        checkit(x1); // 6
        checkit(x2); // 6
    }

    checkit(0);
    checkit(c - 1);
    checkit(r - 1);
    checkit(r + c - 2);

    ys_odd.clear();
    ys_even.clear();
    for (auto &e : evs) {
        if (e.y1 & 1) {
            ys_odd.push_back(e.y1);
            ys_even.push_back(e.y1 + 1);
        } else {
            ys_even.push_back(e.y1);
            ys_odd.push_back(e.y1 + 1);
        }
        if (e.y2 & 1) {
            ys_odd.push_back(e.y2);
            ys_even.push_back(e.y2 + 1);
        } else {
            ys_even.push_back(e.y2);
            ys_odd.push_back(e.y2 + 1);
        }
    }

    sort(ys_odd.begin(), ys_odd.end());
    ys_odd.resize(unique(ys_odd.begin(), ys_odd.end()) - ys_odd.begin());

    sort(ys_even.begin(), ys_even.end());
    ys_even.resize(unique(ys_even.begin(), ys_even.end()) - ys_even.begin());

    init();
    sz[0] = ys_even.size() + 1;
    sz[1] = ys_odd.size() + 1;

    sort(evs.begin(), evs.end(), cmp);

    bool ok = true;
    for (auto &e : evs) {
        if (e.type == -1) {
            add_all(e.y1, e.y2, 1);
        } else if (e.type == 1) {
            add_all(e.y1, e.y2, -1);
        } else {
            if (e.x & 1) {
                if (get_odd(e.y1, e.y2) == 0) {
                    ok = false;
//                    return false;
                }
            } else {
                if (get_even(e.y1, e.y2) == 0) {
                    ok = false;
//                    return false;
                }
            }

        }
    }

    return ok;
//    return true;
}

int main() {
#ifdef DEBUG
    freopen("input", "r", stdin);
#endif
    ios::sync_with_stdio(0);
    cin.tie(0);

    cin >> r >> c;
    cin >> n;
    x.resize(n);
    y.resize(n);

    for (int i = 0; i < n; i++) {
        cin >> x[i] >> y[i];
        x[i]--;
        y[i]--;
    }

    int L = -1;
    int R = r + c;
    while (R - L > 1) {
        int M = (L + R) / 2;
        if (check(M)) {
            R = M;
        } else {
            L = M;
        }
    }

    cout << R << endl;

    return 0;
}
